<?php

namespace QUI\UserDownloads;

use QUI;
use QUI\ERP\Products\Handler\Fields;
use QUI\ERP\Products\Interfaces\ProductInterface;
use QUI\ERP\Order\AbstractOrder;
use QUI\ERP\Products\Handler\Fields as ProductFields;

/**
 * Class Events
 *
 * Main event handlers for quiqqer/user-downloads
 */
class Events
{
    /**
     * quiqqer/order: onQuiqqerOrderCreated
     *
     * Parse download entries from order and save to database.
     *
     * @param AbstractOrder $Order
     * @return void
     */
    public static function onQuiqqerOrderCreated(AbstractOrder $Order): void
    {
        Handler::createDownloadEntriesFromOrder($Order);
    }

    /**
     * quiqqer/quiqqer/: onPackageSetup
     *
     * @param QUI\Package\Package $Package
     * @return void
     */
    public static function onPackageSetup(QUI\Package\Package $Package): void
    {
        if ($Package->getName() !== 'quiqqer/user-downloads') {
            return;
        }

        try {
            self::createProductFields();
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
        }
    }

    /**
     * Create all fixed product fields that quiqqer/stock-management provides
     *
     * @return void
     * @throws QUI\Exception
     */
    protected static function createProductFields(): void
    {
        $fields = [
            Handler::PRODUCT_FIELD_ID_DOWNLOAD_FILES => [
                'title' => [
                    'de' => 'Download-Dateien',
                    'en' => 'Download files'
                ],
                'type' => ProductFields::TYPE_FOLDER,
                'public' => false,
                'standard' => false,
                'options' => [
                    'autoActivateItems' => false
                ]
            ]
        ];

        $fieldsCreated = false;

        foreach ($fields as $fieldId => $field) {
            try {
                ProductFields::getField($fieldId);
                continue;
            } catch (\Exception) {
                // Field does not exist -> create it
            }

            try {
                ProductFields::createField([
                    'id' => $fieldId,
                    'type' => $field['type'],
                    'titles' => $field['title'],
                    'workingtitles' => $field['title'],
                    'systemField' => 0,
                    'standardField' => !empty($field['standard']) ? 1 : 0,
                    'publicField' => !empty($field['public']) ? 1 : 0,
                    'options' => !empty($field['options']) ? $field['options'] : null
                ]);
            } catch (\Exception $Exception) {
                QUI\System\Log::writeException($Exception);
                continue;
            }

            $fieldsCreated = true;
        }

        if ($fieldsCreated) {
            QUI\Translator::publish('quiqqer/products');
        }
    }

    /**
     * Assign plan product fields to a product
     *
     * @param ProductInterface $Product
     * @return void
     */
    public static function onQuiqqerProductsProductCreate(ProductInterface $Product): void
    {
        if (!($Product instanceof DownloadProductType)) {
            return;
        }

        $fields = [
            Handler::PRODUCT_FIELD_ID_DOWNLOAD_FILES
        ];

        foreach ($fields as $fieldId) {
            try {
                $Field = Fields::getField($fieldId);
                $Product->addOwnField($Field);
            } catch (\Exception $Exception) {
                QUI\System\Log::writeException($Exception);
            }
        }

        try {
            $Product->update(QUI::getUsers()->getSystemUser());
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
        }
    }
}
